@@ -3,7 +3,7 @@
  * Also useful as an example of how to use the img code in your own programs
  */
 
-/* Copyright (C) 1994-2004,2008 Olly Betts
+/* Copyright (C) 1994-2004,2008,2010 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -298,8 +298,8 @@ sketch_label(const img_point *p, const char *s, bool fSurface)
    fprintf(fh, "txt('");
    while (*s) {
       int ch = *s++;
-      if (ch == '\'' || ch == '\\') putc('\\', fh);
-      putc(ch, fh);
+      if (ch == '\'' || ch == '\\') PUTC('\\', fh);
+      PUTC(ch, fh);
    }
    fprintf(fh, "',(%.3f,%.3f))\n", p->x * factor, p->y * factor);
 }
@@ -562,7 +562,7 @@ plt_footer(void)
            min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
            min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
    /* Yucky DOS "end of textfile" marker */
-   putc('\x1a', fh);
+   PUTC('\x1a', fh);
 }
 
 static int dxf_passes[] = { LEGS|STNS|LABELS, 0 };
