@@ -30,6 +30,7 @@
 
 #include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 // For select():
 #ifdef HAVE_SYS_SELECT_H
@@ -39,10 +40,10 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-enum { LOG_RERUN = 1234 };
+enum { LOG_REPROCESS = 1234 };
 
 BEGIN_EVENT_TABLE(CavernLogWindow, wxHtmlWindow)
-    EVT_BUTTON(LOG_RERUN, CavernLogWindow::OnRerun)
+    EVT_BUTTON(LOG_REPROCESS, CavernLogWindow::OnReprocess)
     EVT_BUTTON(wxID_OK, CavernLogWindow::OnOK)
 END_EVENT_TABLE()
 
@@ -52,15 +53,20 @@ static wxString escape_for_shell(wxString s, bool protect_dash = false)
 #ifdef __WXMSW__
     bool needs_quotes = false;
     while (p < s.size()) {
-	if (s[p] == wxT('"')) {
-	    s.insert(p, wxT('\\'));
-	    ++p;
-	    needs_quotes = true;
+	wxChar ch = s[p];
+	if (ch < 127) {
+	    if (ch == wxT('"')) {
+		s.insert(p, 1, wxT('\\'));
+		++p;
+		needs_quotes = true;
+	    } else if (strchr(" <>&|^", ch)) {
+		needs_quotes = true;
+	    }
 	}
 	++p;
     }
     if (needs_quotes) {
-	s.insert(0, wxT('"'));
+	s.insert(0u, 1, wxT('"'));
 	s += wxT('"');
     }
 #else
@@ -73,7 +79,7 @@ static wxString escape_for_shell(wxString s, bool protect_dash = false)
     while (p < s.size()) {
 	// Exclude a few safe characters which are common in filenames
 	if (!isalnum(s[p]) && strchr("/._-", s[p]) == NULL) {
-	    s.insert(p, wxT('\\'));
+	    s.insert(p, 1, wxT('\\'));
 	    ++p;
 	}
 	++p;
@@ -115,35 +121,37 @@ CavernLogWindow::OnLinkClicked(const wxHtmlLinkInfo &link)
 	size_t i = 0;
 	while ((i = cmd.find(wxT('$'), i)) != wxString::npos) {
 	    if (++i >= cmd.size()) break;
-		switch (cmd[i]) {
-		    case wxT('$'):
-			cmd.erase(i, 1);
-			break;
-		    case wxT('f'): {
-			wxString f = escape_for_shell(href.substr(0, colon), true);
-			cmd.replace(i - 1, 2, f);
-			i += f.size() - 1;
-			break;
-		    }
-		    case wxT('t'): {
-			wxString t = escape_for_shell(title);
-			cmd.replace(i - 1, 2, t);
-			i += t.size() - 1;
-			break;
-		    }
-		    case wxT('l'): {
-			wxString l = escape_for_shell(href.substr(colon + 1));
-			cmd.replace(i - 1, 2, l);
-			i += l.size() - 1;
-			break;
-		    }
-		    default:
-			++i;
+	    switch (cmd[i]) {
+		case wxT('$'):
+		    cmd.erase(i, 1);
+		    break;
+		case wxT('f'): {
+		    wxString f = escape_for_shell(href.substr(0, colon), true);
+		    cmd.replace(i - 1, 2, f);
+		    i += f.size() - 1;
+		    break;
 		}
+		case wxT('t'): {
+		    wxString t = escape_for_shell(title);
+		    cmd.replace(i - 1, 2, t);
+		    i += t.size() - 1;
+		    break;
+		}
+		case wxT('l'): {
+		    wxString l = escape_for_shell(href.substr(colon + 1));
+		    cmd.replace(i - 1, 2, l);
+		    i += l.size() - 1;
+		    break;
+		}
+		default:
+		    ++i;
+	    }
 	}
-	// FIXME: This should really use fn_str() - currently we probably can't
-	// process a Unicode path on wxmsw.
+#ifdef __WXMSW__
+	_wsystem(cmd.c_str());
+#else
 	system(cmd.mb_str());
+#endif
     }
 }
 
@@ -153,7 +161,6 @@ CavernLogWindow::process(const wxString &file)
     SetFocus();
     filename = file;
 
-    char *cavern = use_path(msg_exepth(), "cavern");
 #ifdef __WXMSW__
     SetEnvironmentVariable(wxT("SURVEX_UTF8"), wxT("1"));
 #else
@@ -161,16 +168,43 @@ CavernLogWindow::process(const wxString &file)
 #endif
 
     wxString escaped_file = escape_for_shell(file, true);
+#ifdef __WXMSW__
+    wxString cmd;
+    {
+	DWORD len = 256;
+	wchar_t *buf = NULL, *p;
+	while (1) {
+	    DWORD got;
+	    buf = (wchar_t*)osrealloc(buf, len * 2);
+	    got = GetModuleFileNameW(NULL, buf, len);
+	    if (got < len) break;
+	    len += len;
+	}
+	/* Strange Win32 nastiness - strip prefix "\\?\" if present */
+	if (wcsncmp(buf, L"\\\\?\\", 4) == 0) buf += 4;
+	p = wcsrchr(buf, L'\\');
+	wchar_t * slash = wcsrchr(buf, L'\\');
+	if (slash) {
+	    cmd.assign(buf, slash - buf + 1);
+	}
+    }
+    cmd += L"cavern";
+    cmd = escape_for_shell(cmd, false);
+#else
+    char *cavern = use_path(msg_exepth(), "cavern");
     wxString cmd = escape_for_shell(wxString(cavern, wxConvUTF8), false);
     osfree(cavern);
+#endif
     cmd += wxT(" -o ");
     cmd += escaped_file;
     cmd += wxT(' ');
     cmd += escaped_file;
 
-    // FIXME: This should really use fn_str() - currently we probably can't
-    // process a Unicode path on wxmsw.
+#ifdef __WXMSW__
+    FILE * cavern_out = _wpopen(cmd.c_str(), L"r");
+#else
     FILE * cavern_out = popen(cmd.mb_str(), "r");
+#endif
     if (!cavern_out) {
 	wxString m;
 	m.Printf(wmsg(/*Couldn't open pipe: `%s'*/17), cmd.c_str());
@@ -199,13 +233,21 @@ CavernLogWindow::process(const wxString &file)
 	FD_SET(cavern_fd, &rfds);
 	FD_ZERO(&efds);
 	FD_SET(cavern_fd, &efds);
-	// Set timeout to 0.1 seconds.
+	// Timeout instantly.
 	struct timeval timeout;
 	timeout.tv_sec = 0;
-	timeout.tv_usec = 100000;
+	timeout.tv_usec = 0;
 	if (select(cavern_fd + 1, &rfds, NULL, &efds, &timeout) == 0) {
-	    wxYield();
-	    continue;
+	    Update();
+	    FD_SET(cavern_fd, &rfds);
+	    FD_SET(cavern_fd, &efds);
+	    // Set timeout to 0.1 seconds.
+	    timeout.tv_sec = 0;
+	    timeout.tv_usec = 100000;
+	    if (select(cavern_fd + 1, &rfds, NULL, &efds, &timeout) == 0) {
+		wxYield();
+		continue;
+	    }
 	}
 	if (!FD_ISSET(cavern_fd, &rfds)) {
 	    // Error, which pclose() should report.
@@ -213,7 +255,7 @@ CavernLogWindow::process(const wxString &file)
 	}
 	int ch;
 	if (left == EOF) {
-	    ch = getc(cavern_out);
+	    ch = GETC(cavern_out);
 	    if (ch == EOF) break;
 	} else {
 	    ch = left;
@@ -222,7 +264,7 @@ CavernLogWindow::process(const wxString &file)
 	// Decode UTF-8 first to avoid security issues with <, >, &, etc
 	// encoded using multibyte encodings.
 	if (ch >= 0xc0 && ch < 0xf0) {
-	    int ch1 = getc(cavern_out);
+	    int ch1 = GETC(cavern_out);
 	    if ((ch1 & 0xc0) != 0x80) {
 		left = ch1;
 	    } else if (ch < 0xe0) {
@@ -230,7 +272,7 @@ CavernLogWindow::process(const wxString &file)
 		ch = ((ch & 0x1f) << 6) | (ch1 & 0x3f);
 	    } else {
 		/* 3 byte sequence */
-		int ch2 = getc(cavern_out);
+		int ch2 = GETC(cavern_out);
 		if ((ch2 & 0xc0) != 0x80) {
 		    ungetc(ch2, cavern_out);
 		    left = ch1;
@@ -298,7 +340,6 @@ CavernLogWindow::process(const wxString &file)
 		}
 
 		cur.clear();
-		Update();
 		break;
 	    }
 	    case '<':
@@ -324,7 +365,7 @@ CavernLogWindow::process(const wxString &file)
 
     int retval = pclose(cavern_out);
     if (retval) {
-	AppendToPage(wxT("<avenbutton default id=1234 name=\"Rerun\">"));
+	AppendToPage(wxT("<avenbutton default id=1234 name=\"Reprocess\">"));
 	if (retval == -1) {
 	    wxString m = wxT("Problem running cavern: ");
 	    m += wxString(strerror(errno), wxConvUTF8);
@@ -334,7 +375,7 @@ CavernLogWindow::process(const wxString &file)
 	return -1;
     }
     if (link_count) {
-	AppendToPage(wxT("<avenbutton id=1234 name=\"Rerun\">"));
+	AppendToPage(wxT("<avenbutton id=1234 name=\"Reprocess\">"));
 	AppendToPage(wxString::Format(wxT("<avenbutton default id=%d>"), (int)wxID_OK));
 	Update();
     }
@@ -342,7 +383,7 @@ CavernLogWindow::process(const wxString &file)
 }
 
 void
-CavernLogWindow::OnRerun(wxCommandEvent & e)
+CavernLogWindow::OnReprocess(wxCommandEvent & e)
 {
     SetPage(wxString());
     if (process(filename) == 0) {
