@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2003,2005,2006 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006,2007 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2007,2010 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 #include <float.h>
 
 #include "aven.h"
+#include "date.h"
 #include "gfxcore.h"
 #include "mainfrm.h"
 #include "message.h"
@@ -400,16 +401,16 @@ void GfxCore::OnPaint(wxPaintEvent&)
 
 	if (!Animating() && (m_here.IsValid() || m_there.IsValid())) {
 	    // Draw "here" and "there".
-	    Double hx, hy;
+	    double hx, hy;
 	    SetColour(HERE_COLOUR);
 	    if (m_here.IsValid()) {
-		Double dummy;
+		double dummy;
 		Transform(m_here, &hx, &hy, &dummy);
 		if (!m_here_is_temporary) DrawRing(hx, hy);
 	    }
 	    if (m_there.IsValid()) {
-		Double tx, ty;
-		Double dummy;
+		double tx, ty;
+		double dummy;
 		Transform(m_there, &tx, &ty, &dummy);
 		if (m_here.IsValid()) {
 		    BeginLines();
@@ -739,7 +740,7 @@ void GfxCore::NattyDrawNames()
 	    continue;
 	}
 
-	Double x, y, z;
+	double x, y, z;
 
 	Transform(**label, &x, &y, &z);
 	// Check if the label is behind us (in perspective view).
@@ -748,7 +749,7 @@ void GfxCore::NattyDrawNames()
 	// Apply a small shift so that translating the view doesn't make which
 	// labels are displayed change as the resulting twinkling effect is
 	// distracting.
-	Double tx, ty, tz;
+	double tx, ty, tz;
 	Transform(Vector3(), &tx, &ty, &tz);
 	tx -= floor(tx / quantise) * quantise;
 	ty -= floor(ty / quantise) * quantise;
@@ -795,7 +796,7 @@ void GfxCore::SimpleDrawNames()
 	    continue;
 	}
 
-	Double x, y, z;
+	double x, y, z;
 	Transform(**label, &x, &y, &z);
 
 	// Check if the label is behind us (in perspective view).
@@ -868,18 +869,13 @@ void GfxCore::DrawDatebar()
     }
 
     wxString* strs = new wxString[GetNumDepthBands()];
-    char buf[128];
     int band;
     for (band = 0; band < GetNumDepthBands(); band++) {
-	time_t date = m_Parent->GetDateMin() +
-		   time_t((double)m_Parent->GetDateExtent() * band / (GetNumDepthBands() - 1));
-	size_t res = strftime(buf, sizeof(buf), "%Y-%m-%d", gmtime(&date));
-	if (res == 0 || res == sizeof(buf)) {
-	    // Insert extra "" to avoid trigraphs issues.
-	    strs[band] = wxT("?""?""?""?-?""?-?""?");
-	} else {
-	    strs[band] = wxString::FromAscii(buf);
-	}
+	int y, m, d;
+	int days = m_Parent->GetDateMin();
+	days += m_Parent->GetDateExtent() * band / (GetNumDepthBands() - 1);
+	ymd_from_days_since_1900(days, &y, &m, &d);
+	strs[band] = wxString::Format(wxT("%04d-%02d-%02d"), y, m, d);
 
 	int x;
 	GetTextExtent(strs[band], &x, NULL);
@@ -1136,7 +1132,7 @@ bool GfxCore::CheckHitTestGrid(const wxPoint& point, bool centre)
     while (iter != m_PointGrid[square].end()) {
 	LabelInfo *pt = *iter++;
 
-	Double cx, cy, cz;
+	double cx, cy, cz;
 
 	Transform(*pt, &cx, &cy, &cz);
 
@@ -1170,7 +1166,7 @@ bool GfxCore::CheckHitTestGrid(const wxPoint& point, bool centre)
 	if (centre) {
 	    ClearTreeSelection();
 	} else {
-	    Double x, y, z;
+	    double x, y, z;
 	    ReverseTransform(point.x, m_YSize - point.y, &x, &y, &z);
 	    SetHere(Point(Vector3(x, y, z)));
 	    m_here_is_temporary = true;
@@ -1423,7 +1419,7 @@ void GfxCore::RefreshLine(const Point &a, const Point &b, const Point &c)
     // Calculate the minimum rectangle which includes the old and new
     // measuring lines to minimise the redraw time
     int l = INT_MAX, r = INT_MIN, u = INT_MIN, d = INT_MAX;
-    Double X, Y, Z;
+    double X, Y, Z;
     if (a.IsValid()) {
 	if (!Transform(a, &X, &Y, &Z)) {
 	    printf("oops\n");
@@ -1517,7 +1513,7 @@ void GfxCore::CreateHitTestGrid()
 	}
 
 	// Calculate screen coordinates.
-	Double cx, cy, cz;
+	double cx, cy, cz;
 	Transform(*label, &cx, &cy, &cz);
 	if (cx < 0 || cx >= m_XSize) continue;
 	if (cy < 0 || cy >= m_YSize) continue;
@@ -1617,7 +1613,7 @@ void GfxCore::SetCoords(wxPoint point)
     // window coordinates.  The relevant display is updated depending on
     // whether we're in plan or elevation view.
 
-    Double cx, cy, cz;
+    double cx, cy, cz;
 
     SetDataTransform();
     ReverseTransform(point.x, m_YSize - 1 - point.y, &cx, &cy, &cz);
@@ -2394,18 +2390,18 @@ void GfxCore::AddQuadrilateralDepth(const Vector3 &a, const Vector3 &b,
     EndPolygon();
 }
 
-void GfxCore::SetColourFromDate(time_t date, Double factor)
+void GfxCore::SetColourFromDate(int date, Double factor)
 {
     // Set the drawing colour based on a date.
 
-    if (date == 0) {
+    if (date == -1) {
 	SetColour(GetSurfacePen(), factor);
 	return;
     }
 
-    time_t date_ext = m_Parent->GetDateExtent();
+    int date_ext = m_Parent->GetDateExtent();
     assert(date_ext > 0);
-    time_t date_offset = date - m_Parent->GetDateMin();
+    int date_offset = date - m_Parent->GetDateMin();
 
     Double how_far = (Double)date_offset / date_ext;
     assert(how_far >= 0.0);
@@ -2438,12 +2434,12 @@ void GfxCore::AddPolylineDate(const traverse & centreline)
     BeginPolyline();
     vector<PointInfo>::const_iterator i, prev_i;
     i = centreline.begin();
-    time_t date = i->GetDate();
+    int date = i->GetDate();
     SetColourFromDate(date, 1.0);
     PlaceVertex(*i);
     prev_i = i;
     while (++i != centreline.end()) {
-	time_t newdate = i->GetDate();
+	int newdate = i->GetDate();
 	if (newdate != date) {
 	    EndPolyline();
 	    BeginPolyline();
@@ -2457,7 +2453,7 @@ void GfxCore::AddPolylineDate(const traverse & centreline)
     EndPolyline();
 }
 
-static time_t static_date_hack; // FIXME
+static int static_date_hack; // FIXME
 
 void GfxCore::AddQuadrilateralDate(const Vector3 &a, const Vector3 &b,
 				   const Vector3 &c, const Vector3 &d)
