@@ -50,6 +50,20 @@
 #ifndef GL_COORD_REPLACE
 #define GL_COORD_REPLACE 0x8862
 #endif
+// GL_POINT_SIZE_RANGE is deprecated in OpenGL 1.2 and later, and replaced by
+// GL_SMOOTH_POINT_SIZE_RANGE.
+#ifndef GL_SMOOTH_POINT_SIZE_RANGE
+#define GL_SMOOTH_POINT_SIZE_RANGE GL_POINT_SIZE_RANGE
+#endif
+// GL_POINT_SIZE_GRANULARITY is deprecated in OpenGL 1.2 and later, and
+// replaced by GL_SMOOTH_POINT_SIZE_GRANULARITY.
+#ifndef GL_SMOOTH_POINT_SIZE_GRANULARITY
+#define GL_SMOOTH_POINT_SIZE_GRANULARITY GL_POINT_SIZE_GRANULARITY
+#endif
+// GL_ALIASED_POINT_SIZE_RANGE was added in OpenGL 1.2.
+#ifndef GL_ALIASED_POINT_SIZE_RANGE
+#define GL_ALIASED_POINT_SIZE_RANGE 0x846D
+#endif
 
 #ifndef USE_FNT
 // Some WIN32 stupidity which causes mingw to fail to link for some reason;
@@ -70,7 +84,7 @@
 
 using namespace std;
 
-const double BLOB_DIAMETER = 5.0;
+const int BLOB_DIAMETER = 5;
 
 static bool opengl_initialised = false;
 
@@ -101,18 +115,23 @@ string GetGLSystemDescription()
     GLint max_viewport[2];
     glGetIntegerv(GL_MAX_VIEWPORT_DIMS, max_viewport);
     GLdouble point_size_range[2];
-    glGetDoublev(GL_POINT_SIZE_RANGE, point_size_range);
+    glGetDoublev(GL_SMOOTH_POINT_SIZE_RANGE, point_size_range);
     GLdouble point_size_granularity;
-    glGetDoublev(GL_POINT_SIZE_GRANULARITY, &point_size_granularity);
+    glGetDoublev(GL_SMOOTH_POINT_SIZE_GRANULARITY, &point_size_granularity);
     info += string_format("R%dG%dB%d\n"
 	     "Max Texture size: %dx%d\n"
 	     "Max Viewport size: %dx%d\n"
-	     "Point Size %.3f-%.3f (granularity %.3f)",
+	     "Smooth Point Size %.3f-%.3f (granularity %.3f)",
 	     (int)red, (int)green, (int)blue,
 	     (int)max_texture_size, (int)max_texture_size,
 	     (int)max_viewport[0], (int)max_viewport[1],
 	     point_size_range[0], point_size_range[1],
 	     point_size_granularity);
+    glGetDoublev(GL_ALIASED_POINT_SIZE_RANGE, point_size_range);
+    if (glGetError() != GL_INVALID_ENUM) {
+	info += string_format("\nAliased point size %.3f-%.3f",
+			      point_size_range[0], point_size_range[1]);
+    }
 
     const GLubyte* gl_extensions = glGetString(GL_EXTENSIONS);
     if (*gl_extensions) {
@@ -223,7 +242,7 @@ const int GLACanvas::m_FontSize = 10;
 // Pass wxWANTS_CHARS so that the window gets cursor keys on MS Windows.
 GLACanvas::GLACanvas(wxWindow* parent, int id)
     : wxGLCanvas(parent, id, wxDefaultPosition, wxDefaultSize, wxWANTS_CHARS),
-      m_Translation()
+      m_Translation(), blob_method(UNKNOWN), cross_method(UNKNOWN)
 {
     // Constructor.
 
@@ -239,6 +258,12 @@ GLACanvas::GLACanvas(wxWindow* parent, int id)
     m_Fog = false;
     m_AntiAlias = false;
     list_flags = 0;
+
+#ifndef USE_FNT
+    int argc = 1;
+    char * argv[] = { (char*)"aven", NULL };
+    glutInit(&argc, argv);
+#endif
 }
 
 GLACanvas::~GLACanvas()
@@ -255,6 +280,29 @@ void GLACanvas::FirstShow()
 {
     SetCurrent();
     opengl_initialised = true;
+    save_hints = false;
+    vendor = wxString((const char *)glGetString(GL_VENDOR), wxConvUTF8);
+    renderer = wxString((const char *)glGetString(GL_RENDERER), wxConvUTF8);
+    wxConfigBase * cfg = wxConfigBase::Get();
+    {
+	wxString s;
+	if (cfg->Read(wxT("opengl_vendor"), &s, wxString()) && s == vendor &&
+	    cfg->Read(wxT("opengl_renderer"), &s, wxString()) && s == renderer) {
+	    // The vendor and renderer are the same as the values we have cached,
+	    // so use the hints we have cached.
+	    int v;
+	    if (cfg->Read(wxT("blob_method"), &v, 0) &&
+		(v == POINT || v == LINES)) {
+		// How to draw blobs.
+		blob_method = v;
+	    }
+	    if (cfg->Read(wxT("cross_method"), &v, 0) &&
+		(v == SPRITE || v == LINES)) {
+		// How to draw crosses.
+		cross_method = v;
+	    }
+	}
+    }
 
     // Clear any cached OpenGL lists.
     vector<GLAList>::iterator i;
@@ -309,13 +357,21 @@ void GLACanvas::FirstShow()
     m_Font.load(path.mb_str());
 #endif
 
-    // Check if we can use GL_POINTS to plot blobs at stations.
-    GLdouble point_size_range[2];
-    glGetDoublev(GL_POINT_SIZE_RANGE, point_size_range);
-    CHECK_GL_ERROR("FirstShow", "glGetDoublev GL_POINT_SIZE_RANGE");
-    glpoint_ok = (point_size_range[0] <= BLOB_DIAMETER &&
-		  point_size_range[1] >= BLOB_DIAMETER);
-    if (glpoint_ok) {
+    if (blob_method == UNKNOWN) {
+	// Check if we can use GL_POINTS to plot blobs at stations.
+	GLdouble point_size_range[2];
+	glGetDoublev(GL_SMOOTH_POINT_SIZE_RANGE, point_size_range);
+	CHECK_GL_ERROR("FirstShow", "glGetDoublev GL_SMOOTH_POINT_SIZE_RANGE");
+	if (point_size_range[0] <= BLOB_DIAMETER &&
+	    point_size_range[1] >= BLOB_DIAMETER) {
+	    blob_method = POINT;
+	} else {
+	    blob_method = LINES;
+	}
+	save_hints = true;
+    }
+
+    if (blob_method == POINT) {
 	glPointSize(BLOB_DIAMETER);
 	CHECK_GL_ERROR("FirstShow", "glPointSize");
     }
@@ -330,47 +386,53 @@ void GLACanvas::FirstShow()
     // The symbolic constants GL_POINT_SPRITE, GL_POINT_SPRITE_ARB, and
     // GL_POINT_SPRITE_NV all give the same number so it doesn't matter
     // which we use.
-    float maxSize = 0.0f;
-    glGetFloatv(GL_POINT_SIZE_MAX, &maxSize);
-    if (maxSize >= 8) {
-	glpoint_sprite = (atoi((const char *)glGetString(GL_VERSION)) >= 2);
-	if (!glpoint_sprite) {
-	    const char * p = (const char *)glGetString(GL_EXTENSIONS);
-	    while (true) {
-		size_t l = 0;
-		if (memcmp(p, "GL_ARB_point_sprite", 19) == 0) {
-		    l = 19;
-		} else if (memcmp(p, "GL_NV_point_sprite", 18) == 0) {
-		    l = 18;
-		}
-		if (l) {
-		    p += l;
-		    if (*p == '\0' || *p == ' ') {
-			glpoint_sprite = true;
-			break;
+    if (cross_method == UNKNOWN) {
+	bool glpoint_sprite = false;
+	float maxSize = 0.0f;
+	glGetFloatv(GL_POINT_SIZE_MAX, &maxSize);
+	if (maxSize >= 8) {
+	    glpoint_sprite = (atoi((const char *)glGetString(GL_VERSION)) >= 2);
+	    if (!glpoint_sprite) {
+		const char * p = (const char *)glGetString(GL_EXTENSIONS);
+		while (true) {
+		    size_t l = 0;
+		    if (memcmp(p, "GL_ARB_point_sprite", 19) == 0) {
+			l = 19;
+		    } else if (memcmp(p, "GL_NV_point_sprite", 18) == 0) {
+			l = 18;
 		    }
+		    if (l) {
+			p += l;
+			if (*p == '\0' || *p == ' ') {
+			    glpoint_sprite = true;
+			    break;
+			}
+		    }
+		    p = strchr(p + 1, ' ');
+		    if (!p) break;
+		    ++p;
 		}
-		p = strchr(p + 1, ' ');
-		if (!p) break;
-		++p;
 	    }
 	}
+	cross_method = glpoint_sprite ? SPRITE : LINES;
+	save_hints = true;
     }
 
-    if (glpoint_sprite) {
+    if (cross_method == SPRITE) {
 	glGenTextures(1, &m_CrossTexture);
 	CHECK_GL_ERROR("FirstShow", "glGenTextures");
 	glBindTexture(GL_TEXTURE_2D, m_CrossTexture);
 	CHECK_GL_ERROR("FirstShow", "glBindTexture");
+	// Cross image for drawing crosses using texture mapped point sprites.
 	const unsigned char crossteximage[128] = {
-	    255,255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,255,
-	      0,  0,255,255,  0,  0,  0,  0,  0,  0,  0,  0,255,255,  0,  0,
-	      0,  0,  0,  0,255,255,  0,  0,  0,  0,255,255,  0,  0,  0,  0,
-	      0,  0,  0,  0,  0,  0,255,255,255,255,  0,  0,  0,  0,  0,  0,
-	      0,  0,  0,  0,  0,  0,255,255,255,255,  0,  0,  0,  0,  0,  0,
-	      0,  0,  0,  0,255,255,  0,  0,  0,  0,255,255,  0,  0,  0,  0,
-	      0,  0,255,255,  0,  0,  0,  0,  0,  0,  0,  0,255,255,  0,  0,
-	    255,255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,255
+	      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+	    255,255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,255,  0,  0,
+	      0,  0,255,255,  0,  0,  0,  0,  0,  0,255,255,  0,  0,  0,  0,
+	      0,  0,  0,  0,255,255,  0,  0,255,255,  0,  0,  0,  0,  0,  0,
+	      0,  0,  0,  0,  0,  0,255,255,  0,  0,  0,  0,  0,  0,  0,  0,
+	      0,  0,  0,  0,255,255,  0,  0,255,255,  0,  0,  0,  0,  0,  0,
+	      0,  0,255,255,  0,  0,  0,  0,  0,  0,255,255,  0,  0,  0,  0,
+	    255,255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,255,  0,  0
 	};
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 	CHECK_GL_ERROR("FirstShow", "glPixelStorei");
@@ -387,8 +449,6 @@ void GLACanvas::FirstShow()
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	CHECK_GL_ERROR("FirstShow", "glTexParameteri GL_TEXTURE_MIN_FILTER");
     }
-    //if (glpoint_ok) printf("Using GL_POINTS for blobs\n");
-    //if (glpoint_sprite) printf("Using GL_POINT_SPRITE* for crosses");
 }
 
 void GLACanvas::Clear()
@@ -418,8 +478,8 @@ void GLACanvas::AddTranslationScreenCoordinates(int dx, int dy)
     // Find out how far the translation takes us in data coordinates.
     SetDataTransform();
 
-    Double x0, y0, z0;
-    Double x, y, z;
+    double x0, y0, z0;
+    double x, y, z;
     gluUnProject(0.0, 0.0, 0.0, modelview_matrix, projection_matrix, viewport,
                  &x0, &y0, &z0);
     CHECK_GL_ERROR("AddTranslationScreenCoordinates", "gluUnProject");
@@ -436,7 +496,7 @@ void GLACanvas::SetVolumeDiameter(glaCoord diameter)
     // Set the size of the data drawing volume by giving the diameter of the
     // smallest sphere containing it.
 
-    m_VolumeDiameter = max(1.0, diameter);
+    m_VolumeDiameter = max(glaCoord(1.0), diameter);
 }
 
 void GLACanvas::StartDrawing()
@@ -444,7 +504,71 @@ void GLACanvas::StartDrawing()
     // Prepare for a redraw operation.
 
     SetCurrent();
-    glDepthMask(true);
+    glDepthMask(GL_TRUE);
+
+    if (!save_hints) return;
+
+    // We want to check on the second redraw.
+    static int draw_count = 2;
+    if (--draw_count != 0) return;
+
+    if (cross_method != LINES) {
+	SetColour(col_WHITE);
+	Clear();
+	SetDataTransform();
+	BeginCrosses();
+	DrawCross(-m_Translation.GetX(), -m_Translation.GetY(), -m_Translation.GetZ());
+	EndCrosses();
+	static const unsigned char expected_cross[64 * 3] = {
+#define o 0,0,0
+#define I 255,255,255
+	    o, o, o, o, o, o, o, o,
+	    o, I, o, o, o, o, I, o,
+	    o, o, I, o, o, I, o, o,
+	    o, o, o, I, I, o, o, o,
+	    o, o, o, I, I, o, o, o,
+	    o, o, I, o, o, I, o, o,
+	    o, I, o, o, o, o, I, o,
+	    o, o, o, o, o, o, o, o
+	};
+	if (!CheckVisualFidelity(expected_cross)) {
+	    cross_method = LINES;
+	    save_hints = true;
+	}
+    }
+
+    if (blob_method != LINES) {
+	SetColour(col_WHITE);
+	Clear();
+	SetDataTransform();
+	BeginBlobs();
+	DrawBlob(-m_Translation.GetX(), -m_Translation.GetY(), -m_Translation.GetZ());
+	EndBlobs();
+	static const unsigned char expected_blob[64 * 3] = {
+#define o 0,0,0
+#define I 255,255,255
+	    o, o, o, o, o, o, o, o,
+	    o, o, o, o, o, o, o, o,
+	    o, o, I, I, I, o, o, o,
+	    o, I, I, I, I, I, o, o,
+	    o, I, I, I, I, I, o, o,
+	    o, I, I, I, I, I, o, o,
+	    o, o, I, I, I, o, o, o,
+	    o, o, o, o, o, o, o, o
+	};
+	if (!CheckVisualFidelity(expected_blob)) {
+	    blob_method = LINES;
+	    save_hints = true;
+	}
+    }
+
+    wxConfigBase * cfg = wxConfigBase::Get();
+    cfg->Write(wxT("opengl_vendor"), vendor);
+    cfg->Write(wxT("opengl_renderer"), renderer);
+    cfg->Write(wxT("blob_method"), blob_method);
+    cfg->Write(wxT("cross_method"), cross_method);
+    cfg->Flush();
+    save_hints = false;
 }
 
 void GLACanvas::EnableSmoothPolygons(bool filled)
@@ -570,13 +694,13 @@ void GLACanvas::SetDataTransform()
 
     if (!m_Perspective) {
 	// Adjust the translation so we don't change the Z position of the model
-	Double X, Y, Z;
+	double X, Y, Z;
 	gluProject(m_Translation.GetX(),
 		   m_Translation.GetY(),
 		   m_Translation.GetZ(),
 		   modelview_matrix, projection_matrix, viewport,
 		   &X, &Y, &Z);
-	Double Tx, Ty, Tz;
+	double Tx, Ty, Tz;
 	gluUnProject(X, Y, 0.5, modelview_matrix, projection_matrix, viewport,
 		     &Tx, &Ty, &Tz);
 	glTranslated(Tx, Ty, Tz);
@@ -797,7 +921,9 @@ void GLACanvas::DrawText(glaCoord x, glaCoord y, glaCoord z, const wxString& str
     glRasterPos3d(x, y, z);
     CHECK_GL_ERROR("DrawText", "glRasterPos3d");
 
-#ifdef FREEGLUT
+// glutBitmapString() useless for Unicode strings.
+//#ifdef FREEGLUT
+#if 0
     glutBitmapString(m_Font, (const unsigned char *)str.c_str());
     CHECK_GL_ERROR("DrawText", "glutBitmapString");
 #else
@@ -831,7 +957,8 @@ void GLACanvas::GetTextExtent(const wxString& str, int * x_ext, int * y_ext)
     m_Font.getTextExtent(str.c_str(), x_ext, y_ext);
 #else
     if (x_ext) {
-#if 1
+// glutBitmapLength() useless for Unicode strings.
+#if 0
 	*x_ext = glutBitmapLength(m_Font, (const unsigned char *)str.c_str());
 	CHECK_GL_ERROR("GetTextExtent", "glutBitmapLength");
 #else
@@ -955,8 +1082,8 @@ void GLACanvas::PlaceIndicatorVertex(glaCoord x, glaCoord y)
 
 void GLACanvas::BeginBlobs()
 {
-    if (glpoint_ok) {
-	// Commence drawing of a set of blobs.
+    // Commence drawing of a set of blobs.
+    if (blob_method == POINT) {
 	glPushAttrib(GL_ENABLE_BIT);
 	CHECK_GL_ERROR("BeginBlobs", "glPushAttrib");
 	glEnable(GL_ALPHA_TEST);
@@ -970,15 +1097,18 @@ void GLACanvas::BeginBlobs()
 	SetIndicatorTransform();
 	glEnable(GL_DEPTH_TEST);
 	CHECK_GL_ERROR("BeginBlobs", "glEnable GL_DEPTH_TEST");
+	glBegin(GL_LINES);
     }
 }
 
 void GLACanvas::EndBlobs()
 {
-    if (glpoint_ok) {
-	// Finish drawing of a set of blobs.
-	glEnd();
-	CHECK_GL_ERROR("EndBlobs", "GL_POINTS");
+    // Finish drawing of a set of blobs.
+    glEnd();
+    if (blob_method == POINT) {
+	CHECK_GL_ERROR("EndBlobs", "glEnd GL_POINTS");
+    } else {
+	CHECK_GL_ERROR("EndBlobs", "glEnd GL_LINES");
     }
     glPopAttrib();
     CHECK_GL_ERROR("EndBlobs", "glPopAttrib");
@@ -986,11 +1116,11 @@ void GLACanvas::EndBlobs()
 
 void GLACanvas::DrawBlob(glaCoord x, glaCoord y, glaCoord z)
 {
-    if (glpoint_ok) {
+    if (blob_method == POINT) {
 	// Draw a marker.
 	PlaceVertex(x, y, z);
     } else {
-	Double X, Y, Z;
+	double X, Y, Z;
 	if (!Transform(Vector3(x, y, z), &X, &Y, &Z)) {
 	    printf("bad transform\n");
 	    return;
@@ -999,14 +1129,19 @@ void GLACanvas::DrawBlob(glaCoord x, glaCoord y, glaCoord z)
 	// but we can save effort with a cheap check here.
 	if (Z <= 0) return;
 
-	// Draw an filled circle.
-	assert(m_Quadric);
-	glTranslated(X, Y, Z);
-	CHECK_GL_ERROR("DrawBlob", "glTranslated 1");
-	gluDisk(m_Quadric, 0, BLOB_DIAMETER * 0.5, 8, 1);
-	CHECK_GL_ERROR("DrawBlob", "gluDisk");
-	glTranslated(-X, -Y, -Z);
-	CHECK_GL_ERROR("DrawBlob", "glTranslated 2");
+	X -= BLOB_DIAMETER * 0.5;
+	Y -= BLOB_DIAMETER * 0.5;
+
+	PlaceVertex(X, Y + 1, Z);
+	PlaceVertex(X, Y + (BLOB_DIAMETER - 1), Z);
+
+	for (int i = 1; i < (BLOB_DIAMETER - 1); ++i) {
+	    PlaceVertex(X + i, Y, Z);
+	    PlaceVertex(X + i, Y + BLOB_DIAMETER, Z);
+	}
+
+	PlaceVertex(X + (BLOB_DIAMETER - 1), Y + 1, Z);
+	PlaceVertex(X + (BLOB_DIAMETER - 1), Y + (BLOB_DIAMETER - 1), Z);
     }
 #ifdef GLA_DEBUG
     m_Vertices++;
@@ -1015,18 +1150,23 @@ void GLACanvas::DrawBlob(glaCoord x, glaCoord y, glaCoord z)
 
 void GLACanvas::DrawBlob(glaCoord x, glaCoord y)
 {
-    if (glpoint_ok) {
+    if (blob_method == POINT) {
 	// Draw a marker.
 	PlaceVertex(x, y, 0);
     } else {
-	// Draw an filled circle.
-	assert(m_Quadric);
-	glTranslated(x, y, 0);
-	CHECK_GL_ERROR("DrawBlob 2", "glTranslated 1");
-	gluDisk(m_Quadric, 0, BLOB_DIAMETER * 0.5, 8, 1);
-	CHECK_GL_ERROR("DrawBlob 2", "gluDisk");
-	glTranslated(-x, -y, 0);
-	CHECK_GL_ERROR("DrawBlob 2", "glTranslated 2");
+	x -= BLOB_DIAMETER * 0.5;
+	y -= BLOB_DIAMETER * 0.5;
+
+	PlaceVertex(x, y + 1, 0);
+	PlaceVertex(x, y + (BLOB_DIAMETER - 1), 0);
+
+	for (int i = 1; i < (BLOB_DIAMETER - 1); ++i) {
+	    PlaceVertex(x + i, y, 0);
+	    PlaceVertex(x + i, y + BLOB_DIAMETER, 0);
+	}
+
+	PlaceVertex(x + (BLOB_DIAMETER - 1), y + 1, 0);
+	PlaceVertex(x + (BLOB_DIAMETER - 1), y + (BLOB_DIAMETER - 1), 0);
     }
 #ifdef GLA_DEBUG
     m_Vertices++;
@@ -1036,7 +1176,7 @@ void GLACanvas::DrawBlob(glaCoord x, glaCoord y)
 void GLACanvas::BeginCrosses()
 {
     // Plot crosses.
-    if (glpoint_sprite) {
+    if (cross_method == SPRITE) {
 	glPushAttrib(GL_ENABLE_BIT|GL_POINT_BIT);
 	CHECK_GL_ERROR("BeginCrosses", "glPushAttrib");
 	glBindTexture(GL_TEXTURE_2D, m_CrossTexture);
@@ -1069,7 +1209,7 @@ void GLACanvas::BeginCrosses()
 void GLACanvas::EndCrosses()
 {
     glEnd();
-    if (glpoint_sprite) {
+    if (cross_method == SPRITE) {
 	CHECK_GL_ERROR("EndCrosses", "glEnd GL_POINTS");
     } else {
 	CHECK_GL_ERROR("EndCrosses", "glEnd GL_LINES");
@@ -1080,10 +1220,10 @@ void GLACanvas::EndCrosses()
 
 void GLACanvas::DrawCross(glaCoord x, glaCoord y, glaCoord z)
 {
-    if (glpoint_sprite) {
+    if (cross_method == SPRITE) {
 	PlaceVertex(x, y, z);
     } else {
-	Double X, Y, Z;
+	double X, Y, Z;
 	if (!Transform(Vector3(x, y, z), &X, &Y, &Z)) {
 	    printf("bad transform\n");
 	    return;
@@ -1256,7 +1396,7 @@ void GLACanvas::DisableDashedLines()
 }
 
 bool GLACanvas::Transform(const Vector3 & v,
-                          Double* x_out, Double* y_out, Double* z_out) const
+			  double* x_out, double* y_out, double* z_out) const
 {
     // Convert from data coordinates to screen coordinates.
 
@@ -1267,7 +1407,7 @@ bool GLACanvas::Transform(const Vector3 & v,
 }
 
 void GLACanvas::ReverseTransform(Double x, Double y,
-                                 Double* x_out, Double* y_out, Double* z_out) const
+				 double* x_out, double* y_out, double* z_out) const
 {
     // Convert from screen coordinates to data coordinates.
 
@@ -1350,3 +1490,13 @@ bool GLACanvas::SaveScreenshot(const wxString & fnm, int type) const
     wxImage grab(width, height, pixels);
     return grab.SaveFile(fnm, type);
 }
+
+bool GLACanvas::CheckVisualFidelity(const unsigned char * target) const
+{
+    int width;
+    int height;
+    GetSize(&width, &height);
+    unsigned char pixels[3 * 8 * 8];
+    glReadPixels(width / 2 - 3, height / 2 - 4, 8, 8, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid *)pixels);
+    return (memcmp(pixels, target, sizeof(pixels)) == 0);
+}
