@@ -130,12 +130,18 @@ class GLACanvas : public wxGLCanvas {
     bool m_Perspective;
     bool m_Fog;
     bool m_AntiAlias;
-    bool glpoint_ok;
-    bool glpoint_sprite;
+    bool save_hints;
+    enum { UNKNOWN = 0, POINT = 'P', LINES = 'L', SPRITE = 'S' };
+    int blob_method;
+    int cross_method;
 
     vector<GLAList> drawing_lists;
     mutable unsigned int list_flags;
 
+    wxString vendor, renderer;
+
+    bool CheckVisualFidelity(const unsigned char * target) const;
+
 public:
     GLACanvas(wxWindow* parent, int id);
     ~GLACanvas();
@@ -225,8 +231,8 @@ public:
     }
     void AddTranslationScreenCoordinates(int dx, int dy);
 
-    bool Transform(const Vector3 & v, Double* x_out, Double* y_out, Double* z_out) const;
-    void ReverseTransform(Double x, Double y, Double* x_out, Double* y_out, Double* z_out) const;
+    bool Transform(const Vector3 & v, double* x_out, double* y_out, double* z_out) const;
+    void ReverseTransform(Double x, Double y, double* x_out, double* y_out, double* z_out) const;
 
 #ifdef USE_FNT
     int GetFontSize() const { return m_Font.getFontSize(); }
