@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2002,2005,2006 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2010 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -304,10 +304,7 @@ class AvenPresList : public wxListCtrl {
 		wxString ext(wxT("*.fly"));
 #else
 		wxString ext = wmsg(/*Aven presentations*/320);
-		ext += wxT("|*.fly|");
-		ext += wmsg(/*All files*/208);
-		ext += wxT("|");
-		ext += wxFileSelectorDefaultWildcardStr;
+		ext += wxT("|*.fly");
 #endif
 		wxFileDialog dlg(this, wmsg(/*Select an output filename*/319),
 				 wxString(), fnm, ext,
@@ -316,9 +313,11 @@ class AvenPresList : public wxListCtrl {
 		fnm = dlg.GetPath();
 	    }
 
-	    // FIXME: This should really use fn_str() - currently we probably can't
-	    // save to a Unicode path on wxmsw.
+#ifdef __WXMSW__
+	    FILE * fh_pres = _wfopen(fnm.fn_str(), L"w");
+#else
 	    FILE * fh_pres = fopen(fnm.mb_str(), "w");
+#endif
 	    if (!fh_pres) {
 		wxGetApp().ReportError(wxString::Format(wmsg(/*Error writing to file `%s'*/110), fnm.c_str()));
 		return;
@@ -327,21 +326,21 @@ class AvenPresList : public wxListCtrl {
 	    for (i = entries.begin(); i != entries.end(); ++i) {
 		const PresentationMark &p = *i;
 		write_double(p.GetX(), fh_pres);
-		putc(' ', fh_pres);
+		PUTC(' ', fh_pres);
 		write_double(p.GetY(), fh_pres);
-		putc(' ', fh_pres);
+		PUTC(' ', fh_pres);
 		write_double(p.GetZ(), fh_pres);
-		putc(' ', fh_pres);
+		PUTC(' ', fh_pres);
 		write_double(p.angle, fh_pres);
-		putc(' ', fh_pres);
+		PUTC(' ', fh_pres);
 		write_double(p.tilt_angle, fh_pres);
-		putc(' ', fh_pres);
+		PUTC(' ', fh_pres);
 		write_double(p.scale, fh_pres);
 		if (p.time != 0.0) {
-		    putc(' ', fh_pres);
+		    PUTC(' ', fh_pres);
 		    write_double(p.time, fh_pres);
 		}
-		putc('\n', fh_pres);
+		PUTC('\n', fh_pres);
 	    }
 	    fclose(fh_pres);
 	    filename = fnm;
@@ -355,9 +354,11 @@ class AvenPresList : public wxListCtrl {
 	    force_save_as = true;
 	}
 	bool Load(const wxString &fnm) {
-	    // FIXME: This should really use fn_str() - currently we probably
-	    // can't save to a Unicode path on wxmsw.
+#ifdef __WXMSW__
+	    FILE * fh_pres = _wfopen(fnm.fn_str(), L"r");
+#else
 	    FILE * fh_pres = fopen(fnm.mb_str(), "r");
+#endif
 	    if (!fh_pres) {
 		wxString m;
 		m.Printf(wmsg(/*Couldn't open file `%s'*/93), fnm.c_str());
@@ -370,7 +371,7 @@ class AvenPresList : public wxListCtrl {
 		char buf[4096];
 		size_t i = 0;
 		while (i < sizeof(buf) - 1) {
-		    int ch = getc(fh_pres);
+		    int ch = GETC(fh_pres);
 		    if (ch == EOF || ch == '\n' || ch == '\r') break;
 		    buf[i++] = ch;
 		}
@@ -996,15 +997,8 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
     m_DepthMin = DBL_MAX;
     Double depthmax = -DBL_MAX;
 
-    m_DateMin = (time_t)-1;
-    if (m_DateMin < 0) {
-	// Hmm, signed time_t!
-	// FIXME: find a cleaner way to do this...
-	time_t x = time_t(1) << (sizeof(time_t) * 8 - 2);
-	m_DateMin = x;
-	while ((x>>=1) != 0) m_DateMin |= x;
-    }
-    time_t datemax = 0;
+    m_DateMin = INT_MAX;
+    int datemax = 0;
     complete_dateinfo = true;
 
     traverses.clear();
@@ -1056,9 +1050,9 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		if (pt.z < zmin) zmin = pt.z;
 		if (pt.z > zmax) zmax = pt.z;
 
-		time_t date;
-		date = survey->date1 + (survey->date2 - survey->date1) / 2;
-		if (date) {
+		int date = survey->days1;
+		if (date != -1) {
+		    date += (survey->days2 - date) / 2;
 		    if (date < m_DateMin) m_DateMin = date;
 		    if (date > datemax) datemax = date;
 		} else {
@@ -1161,9 +1155,9 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		    break;
 		}
 
-		time_t date;
-		date = survey->date1 + (survey->date2 - survey->date1) / 2;
-		if (date) {
+		int date = survey->days1;
+		if (date != -1) {
+		    date += (survey->days2 - date) / 2;
 		    if (date < m_DateMin) m_DateMin = date;
 		    if (date > datemax) datemax = date;
 		}
@@ -1277,7 +1271,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 
     m_Ext.assign(xmax - xmin, ymax - ymin, zmax - zmin);
 
-    if (datemax < m_DateMin) m_DateMin = 0;
+    if (datemax < m_DateMin) m_DateMin = datemax;
     m_DateExt = datemax - m_DateMin;
 
     // Sort the labels.
