@@ -1,6 +1,6 @@
 /* namecmp.h */
 /* Ordering function for station names */
-/* Copyright (C) 2001,2002 Olly Betts
+/* Copyright (C) 2001,2002,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef __cplusplus
+#include "wx.h"
+
 extern "C" {
 #endif
 
@@ -25,4 +27,8 @@ extern int name_cmp(const char *a, const char *b, int separator);
 
 #ifdef __cplusplus
 };
+
+inline int name_cmp(const wxString &a, const wxString &b, int separator) {
+    return name_cmp(a.mb_str(), b.mb_str(), separator);
+}
 #endif
