@@ -1,7 +1,7 @@
 /* netskel.c
  * Survex network reduction - remove trailing traverses and concatenate
  * traverses between junctions
- * Copyright (C) 1991-2004,2005,2006 Olly Betts
+ * Copyright (C) 1991-2004,2005,2006,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -444,11 +444,10 @@ replace_travs(void)
 	    img_write_item(pimg, img_MOVE, 0, NULL,
 			   POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
 	    if (leg->meta) {
-		pimg->date1 = leg->meta->date1;
-		pimg->date2 = leg->meta->date2;
+		pimg->days1 = leg->meta->days1;
+		pimg->days2 = leg->meta->days2;
 	    } else {
-		pimg->date1 = 0;
-		pimg->date2 = 0;
+		pimg->days1 = pimg->days2 = -1;
 	    }
 	    img_write_item(pimg, img_LINE, leg->l.flags,
 			   sprint_prefix(stn1->name->up),
@@ -632,11 +631,10 @@ replace_travs(void)
 	    SVX_ASSERT(!fEquate);
 	    SVX_ASSERT(!fZeros(&leg->v));
 	    if (leg->meta) {
-		pimg->date1 = leg->meta->date1;
-		pimg->date2 = leg->meta->date2;
+		pimg->days1 = leg->meta->days1;
+		pimg->days2 = leg->meta->days2;
 	    } else {
-		pimg->date1 = 0;
-		pimg->date2 = 0;
+		pimg->days1 = pimg->days2 = -1;
 	    }
 	    img_write_item(pimg, img_LINE, leg->l.flags,
 			   sprint_prefix(leg_pfx),
@@ -804,11 +802,10 @@ replace_trailing_travs(void)
 	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
 	    SVX_ASSERT(!fZeros(&leg->v));
 	    if (leg->meta) {
-		pimg->date1 = leg->meta->date1;
-		pimg->date2 = leg->meta->date2;
+		pimg->days1 = leg->meta->days1;
+		pimg->days2 = leg->meta->days2;
 	    } else {
-		pimg->date1 = 0;
-		pimg->date2 = 0;
+		pimg->days1 = pimg->days2 = -1;
 	    }
 	    img_write_item(pimg, img_LINE, leg->l.flags,
 			   sprint_prefix(leg_pfx),
@@ -842,11 +839,10 @@ replace_trailing_travs(void)
       img_write_item(pimg, img_MOVE, 0, NULL,
 		     POS(p->fr, 0), POS(p->fr, 1), POS(p->fr, 2));
       if (p->meta) {
-	  pimg->date1 = p->meta->date1;
-	  pimg->date2 = p->meta->date2;
+	  pimg->days1 = p->meta->days1;
+	  pimg->days2 = p->meta->days2;
       } else {
-	  pimg->date1 = 0;
-	  pimg->date2 = 0;
+	  pimg->days1 = pimg->days2 = -1;
       }
       img_write_item(pimg, img_LINE, p->flags,
 		     sprint_prefix(p->fr->name->up),
@@ -967,11 +963,10 @@ write_passage_models(void)
 	 pimg->u = xsect->u;
 	 pimg->d = xsect->d;
 	 if (xsect->meta) {
-	     pimg->date1 = xsect->meta->date1;
-	     pimg->date2 = xsect->meta->date2;
+	     pimg->days1 = xsect->meta->days1;
+	     pimg->days2 = xsect->meta->days2;
 	 } else {
-	     pimg->date1 = 0;
-	     pimg->date2 = 0;
+	     pimg->days1 = pimg->days2 = -1;
 	 }
 
 	 pfx = xsect->stn;
