@@ -1,6 +1,6 @@
 /* printwx.cc */
 /* wxWidgets specific parts of Survex wxWidgets printing code */
-/* Copyright (C) 1993-2003,2004,2005,2006 Olly Betts
+/* Copyright (C) 1993-2003,2004,2005,2006,2010 Olly Betts
  * Copyright (C) 2001,2004 Philip Underwood
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,6 @@ using namespace std;
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
-#include <time.h>
 #include <ctype.h>
 #include <float.h>
 #include <limits.h>
@@ -684,7 +683,7 @@ svxPrintout::draw_info_box()
     * mis-interpreted as us claiming copyright on the survey, so let's
     * give the website URL instead */
    MOVEMM(boxwidth + 2, 2);
-   WriteString(wxT("Survex "VERSION" - http://www.survex.com/"));
+   WriteString(wxT("Survex "VERSION" - http://survex.com/"));
 
    draw_scale_bar(boxwidth + 10.0, 17.0, l->PaperWidth - boxwidth - 18.0);
 }
